<?php
/**
 * Create an audio version of your posts, with a selection of more than 275+ voices across more than 48 languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/speaker
 *
 * @encoding        UTF-8
 * @version         3.3.2
 * @copyright       Copyright (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Speaker;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Google\Exception;
use Google\Service\Drive;
use Google_Client;
use Merkulove\Speaker;

/**
 * SINGLETON: Class adds admin styles.
 *
 * @since 3.0.0
 *
 **/
final class StorageGoogle {

	/**
	 * The one true StorageGoogle.
	 *
	 * @var StorageGoogle
	 * @since 3.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new StorageGoogle instance.
	 *
	 * @since 3.0.0
	 * @access public
	 **/
	private function __construct()
    {

        /** Catch Google Drive authorization code */
        if ( isset( $_GET[ 'code' ] ) && isset( $_GET[ 'scope' ] ) ) {
            add_action( 'wp_loaded', [ $this, 'gd_catcher' ] );
        }

	}

    /**
     * Generate authorization link for Google Drive
     * @return string
     * @throws Exception
     */
	public function gd_authorization_url()
    {

        $options = Settings::get_instance()->options;
        $authUrl = '';

        $client = $this->get_Google_Client();

        // If there is no previous token or it's expired.
        if ( $client->isAccessTokenExpired() ) {

            // Refresh the token if possible, else fetch a new one.
            if ($client->getRefreshToken()){
                $client->fetchAccessTokenWithRefreshToken($client->getRefreshToken());
            } else {
                // Request authorization from the user.
                $authUrl = $client->createAuthUrl();
            }

        }

        return $authUrl;

    }


    /**
     * Return token link markup
     * @throws Exception
     */
    public function gd_token_link( $key_exist )
    {

        if ( $key_exist ) {

            $token = get_option( 'mdp_speaker_gd_token' );

            ?><p class="mdp-speaker-gb-warn">
            <?php if ( $token ) {

                if ( $this->get_drive_client()->isAccessTokenExpired() ) {
                    ?><span class="material-icons material-icons-outlined material-icons-warn">error</span><?php
                } else {
                    ?><span class="material-icons material-icons-outlined material-icons-ok">check_circle</span><?php
                }

            } else {

                ?><span class="material-icons material-icons-outlined material-icons-warn">error</span><?php

            } ?>
            <a href="<?php echo esc_url( $this->gd_authorization_url() ); ?>" target="_blank" >
                <?php
                if ( $token ) :
                    esc_html_e( 'Re-validate token', 'speaker' );
                else :
                    esc_html_e( 'Get a token', 'speaker' );
                endif;
                ?>
            </a> <?php esc_html_e( 'to link this site to the Google Drive', 'speaker' ); ?>
            </p><?php

        } else {

            delete_option( 'mdp_speaker_gd_token' );

        }

    }

    /**
     * Returns non-authorized API client
     * @return Google_Client
     * @throws Exception
     */
    private function get_Google_Client()
    {

        $client = new Google_Client();
        $client->setApplicationName( Speaker::$name );
        $client->setScopes( [ Drive::DRIVE_METADATA_READONLY, Drive::DRIVE ] );
        $client->setAuthConfig( Settings::get_instance()->options[ 'storage-api-key' ] );
        $client->setAccessType('offline');
        $client->setPrompt('select_account consent');

        return $client;

    }

    /**
     * Returns an authorized API client.
     * @return Google_Client the authorized client object
     * @throws Exception
     */
    public function get_drive_client()
    {

        $access_token = get_option( 'mdp_speaker_gd_token' );

        $client = $this->get_Google_Client();
        $client->setAccessToken( $access_token );

        if ( $client->isAccessTokenExpired() ) {

            if ( $client->getRefreshToken() ) {
                $client->fetchAccessTokenWithRefreshToken( $client->getRefreshToken() );
            }

        }

        return $client;

    }

    /**
     * Store access token
     * @throws Exception
     */
	public function gd_catcher(){

        $auth_code = $_GET[ 'code' ];
        if ( ! is_admin() && empty( $auth_code ) ) { return; }

        $client = $this->get_Google_Client();
        $token = $client->fetchAccessTokenWithAuthCode( $auth_code );

        /** Store token to the DB */
        if ( is_array( $token ) && isset( $token[ 'access_token' ] ) ) {

            update_option( 'mdp_speaker_gd_token', $token ); // TODO: extend token lifetime

        }

    }

    /**
     * Upload to Google Drive
     *
     * @param $file_path
     * @param $post_id
     */
    public function gd_upload_file( $file_path, $post_id ) {

        $options = Settings::get_instance()->options;

        /** Remove token and exit if api key is not exist */
        if ( strlen( $options[ 'storage-api-key' ] ) < 400 ) {

            delete_option( 'mdp_speaker_gd_token' );
            return;

        }

        /** Exit if token is not exist */
        $access_token = get_option( 'mdp_speaker_gd_token' );
        if ( ! $access_token ) { return; }

        /** Get the API client and construct the service object */
        $client = StorageGoogle::get_instance()->get_drive_client();
        $service = new Drive( $client );
        $file = new Drive\DriveFile();

        /** Get folder ID */
        $folder_id = $this->gd_folder_id( $service, $file );

        /** Find folder on the Google Drive */
        $has_folder = $this->gb_folder_exist( $service, $folder_id );

        if ( $has_folder ) {

            $this->gd_create_file( $service, $file, $folder_id, $file_path, $post_id );

        } else {

            echo esc_html__( 'Cannot access Google Drive', 'speaker' );

        }

    }

    /**
     * Create file on the Google Drive
     *
     * @param $service
     * @param $file
     * @param $folder_id
     * @param $file_path
     * @param $post_id
     */
    private function gd_create_file( $service, $file, $folder_id, $file_path, $post_id )
    {

        $options = Settings::get_instance()->options;

        $file_content = file_get_contents( $file_path );
        $file_mime = mime_content_type( $file_path );
        $post_name = get_the_title( $post_id );
        $time = current_time( 'Y-m-d H:i:s' );
        $site = get_bloginfo( 'name' );

        $file->setName( $post_name . ' - ' . $time . '.' . $options[ 'audio-format' ] );
        $file->setDescription( $post_name . ' ' . esc_html__( 'from' ) . ' ' . $site );
        $file->setMimeType( $file_mime );
        $file->setParents( [ $folder_id ] );

        try {

            $results = $service->files->create( $file, array(
                'data' => $file_content,
                'mimeType' => $file_mime,
                'uploadType' => 'multipart'
            ) );

            /** Store audio ID to the post meta */
            if ( isset( $results->id ) ) {

                update_post_meta( $post_id, 'mdp_speaker_gd', $results->id );

            }

        } catch( Exception $e ){

            echo 'An error occurred : ' . $e->getMessage();

        }

    }

    /**
     * Create folder on the Google Drive
     *
     * @param $service
     * @param $file
     * @return string
     */
    private function gd_create_folder( $service, $file )
    {

        $file->setName( 'Speaker - ' . get_bloginfo( 'name' ) );
        $file->setDescription( esc_html__( 'Records created by Speaker WordPress Plugin', 'speaker' ) );
        $file->setMimeType( 'application/vnd.google-apps.folder' );

        $gd_response = $service->files->create( $file );
        $folder_id = $gd_response->id ?? '';

        if ( isset( $gd_response->id ) ) {
            update_option( 'mdp_speaker_gd_folder', $folder_id );
        }

        return $folder_id;

    }

    /**
     * Check if Drive has folder by ID
     *
     * @param $service
     * @param $folder_id
     * @return bool
     */
    private function gb_folder_exist( $service, $folder_id )
    {

        $gd_response = $service->files->get( $folder_id );
        return isset( $gd_response->id );

    }

    /**
     * Get Google Drive folder id for the current site
     *
     * @param $service
     * @param $file
     * @return mixed|string|void
     */
    private function gd_folder_id( $service, $file )
    {

        $folder_id = get_option( 'mdp_speaker_gd_folder' );
        return ! $folder_id ? $this->gd_create_folder( $service, $file ) : $folder_id;

    }

	/**
	 * Main StorageGoogle Instance.
	 *
	 * Insures that only one instance of StorageGoogle exists in memory at any one time.
	 *
	 * @static
	 * @return StorageGoogle
	 * @since 3.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof StorageGoogle ) ) {

			self::$instance = new StorageGoogle;

		}

		return self::$instance;

	}

} // End Class StorageGoogle.
